@@ -83,6 +83,14 @@ class GitlabBuildRetryError(GitlabOperationError):
     pass
 
 
+class GitlabBlockError(GitlabOperationError):
+    pass
+
+
+class GitlabUnblockError(GitlabOperationError):
+    pass
+
+
 def raise_error_from_response(response, error, expected_code=200):
     """Tries to parse gitlab error message from response and raises error.
 
