@@ -34,9 +34,7 @@ from gitlab.exceptions import *  # noqa
 class jsonEncoder(json.JSONEncoder):
     def default(self, obj):
         if isinstance(obj, GitlabObject):
-            return {k: v for k, v in six.iteritems(obj.__dict__)
-                    if (not isinstance(v, BaseManager)
-                        and not k[0] == '_')}
+            return obj.as_dict()
         elif isinstance(obj, gitlab.Gitlab):
             return {'url': obj._url}
         return json.JSONEncoder.default(self, obj)
@@ -157,18 +155,6 @@ class BaseManager(object):
             raise NotImplementedError
         self.gitlab.delete(self.obj_cls, id, **args)
 
-    def _custom_list(self, url, cls, **kwargs):
-        r = self.gitlab._raw_get(url, **kwargs)
-        raise_error_from_response(r, GitlabListError)
-
-        l = []
-        for j in r.json():
-            o = cls(self, j)
-            o._from_api = True
-            l.append(o)
-
-        return l
-
 
 class GitlabObject(object):
     """Base class for all classes that interface with GitLab."""
@@ -179,7 +165,6 @@ class GitlabObject(object):
     _urlPlural = None
     _id_in_delete_url = True
     _id_in_update_url = True
-    _returnClass = None
     _constructorTypes = None
 
     #: Whether _get_list_or_object should return list or object when id is None
@@ -488,6 +473,19 @@ class GitlabObject(object):
         """
         return json.dumps(self, cls=jsonEncoder)
 
+    def as_dict(self):
+        """Dump the object as a dict."""
+        return {k: v for k, v in six.iteritems(self.__dict__)
+                if (not isinstance(v, BaseManager) and not k[0] == '_')}
+
+    def __eq__(self, other):
+        if type(other) is type(self):
+            return self.as_dict() == other.as_dict()
+        return False
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
 
 class UserKey(GitlabObject):
     _url = '/users/%(user_id)s/keys'
@@ -527,10 +525,73 @@ class User(GitlabObject):
                                            user_id=self.id,
                                            **kwargs)
 
+    def block(self, **kwargs):
+        """Blocks the user."""
+        url = '/users/%s/block' % self.id
+        r = self.gitlab._raw_put(url, **kwargs)
+        raise_error_from_response(r, GitlabBlockError)
+        self.state = 'blocked'
+
+    def unblock(self, **kwargs):
+        """Unblocks the user."""
+        url = '/users/%s/unblock' % self.id
+        r = self.gitlab._raw_put(url, **kwargs)
+        raise_error_from_response(r, GitlabUnblockError)
+        self.state = 'active'
+
+    def __eq__(self, other):
+        if type(other) is type(self):
+            selfdict = self.as_dict()
+            otherdict = other.as_dict()
+            selfdict.pop('password', None)
+            otherdict.pop('password', None)
+            return selfdict == otherdict
+        return False
+
 
 class UserManager(BaseManager):
     obj_cls = User
 
+    def search(self, query, **kwargs):
+        """Search users.
+
+        Args:
+            query (str): The query string to send to GitLab for the search.
+            all (bool): If True, return all the items, without pagination
+            **kwargs: Additional arguments to send to GitLab.
+
+        Returns:
+            list(User): A list of matching users.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabListError: If the server fails to perform the request.
+        """
+        url = self.obj_cls._url + '?search=' + query
+        return self.gitlab._raw_list(url, self.obj_cls, **kwargs)
+
+    def get_by_username(self, username, **kwargs):
+        """Get a user by its username.
+
+        Args:
+            username (str): The name of the user.
+            **kwargs: Additional arguments to send to GitLab.
+
+        Returns:
+            User: The matching user.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabGetError: If the server fails to perform the request.
+        """
+        url = self.obj_cls._url + '?username=' + username
+        results = self.gitlab._raw_list(url, self.obj_cls, **kwargs)
+        assert len(results) in (0, 1)
+        try:
+            return results[0]
+        except IndexError:
+            raise GitlabGetError('no such user: ' + username)
+
 
 class CurrentUserKey(GitlabObject):
     _url = '/user/keys'
@@ -640,10 +701,15 @@ class GroupManager(BaseManager):
     def search(self, query, **kwargs):
         """Searches groups by name.
 
-        Returns a list of matching groups.
+        Args:
+            query (str): The search string
+            all (bool): If True, return all the items, without pagination
+
+        Returns:
+            list(Group): a list of matching groups.
         """
         url = '/groups?search=' + query
-        return self._custom_list(url, Group, **kwargs)
+        return self.gitlab._raw_list(url, self.obj_cls, **kwargs)
 
 
 class Hook(GitlabObject):
@@ -851,7 +917,8 @@ class ProjectHook(GitlabObject):
     requiredUrlAttrs = ['project_id']
     requiredCreateAttrs = ['url']
     optionalCreateAttrs = ['push_events', 'issues_events',
-                           'merge_requests_events', 'tag_push_events']
+                           'merge_requests_events', 'tag_push_events',
+                           'build_events', 'enable_ssl_verification']
     shortPrintAttr = 'url'
 
 
@@ -1449,37 +1516,50 @@ class ProjectManager(BaseManager):
     def search(self, query, **kwargs):
         """Search projects by name.
 
+        .. note::
+
+           The search is only performed on the project name (not on the
+           namespace or the description). To perform a smarter search, use the
+           ``search`` argument of the ``list()`` method:
+
+           .. code-block:: python
+
+               gl.projects.list(search=your_search_string)
+
         Args:
             query (str): The query string to send to GitLab for the search.
+            all (bool): If True, return all the items, without pagination
             **kwargs: Additional arguments to send to GitLab.
 
         Returns:
             list(Project): A list of matching projects.
         """
-        return self._custom_list("/projects/search/" + query, Project,
-                                 **kwargs)
+        return self.gitlab._raw_list("/projects/search/" + query, Project,
+                                     **kwargs)
 
     def all(self, **kwargs):
         """List all the projects (need admin rights).
 
         Args:
+            all (bool): If True, return all the items, without pagination
             **kwargs: Additional arguments to send to GitLab.
 
         Returns:
             list(Project): The list of projects.
         """
-        return self._custom_list("/projects/all", Project, **kwargs)
+        return self.gitlab._raw_list("/projects/all", Project, **kwargs)
 
     def owned(self, **kwargs):
         """List owned projects.
 
         Args:
+            all (bool): If True, return all the items, without pagination
             **kwargs: Additional arguments to send to GitLab.
 
         Returns:
             list(Project): The list of owned projects.
         """
-        return self._custom_list("/projects/owned", Project, **kwargs)
+        return self.gitlab._raw_list("/projects/owned", Project, **kwargs)
 
 
 class UserProjectManager(BaseManager):
